/*
    symtest.prg

    This is a test of the Symbol class.  It was designed more for testing
    purposes than tutorial purposes, and does not do anything useful, but
    does demonstrate some of the possibilities of the class.  For a slightly
    more sane explanation of the Symbol class, please see the Norton Guide.
*/

PROCEDURE Main
    // starting off slow...  Create a symbol object referring to the 'qout'
    // symbol.  Note we don't use a character string to do this, but rather
    // an real identifier (ie. 'qout' is not in quotes below).
    LOCAL oSym := Symbol():qout

    // execute the function referred to by the symbol, in other words, QOUT()
    oSym:exec( "Hi there!", "This is pretty neat, huh?" )

    // print out the name of the symbol: "QOUT"
    ? oSym:name

    // Instead of the question mark used above, since the symbol we're
    // playing with is QOUT, we'll use it to print it's own name!
    oSym:exec(oSym:name)

    // following allows nesting testing and parameter
    // integrity checking in the debugger
    oSym := Symbol():WhatFun
    ? oSym:name, "returned:", oSym:exec( 99, 88, 77 )

    // finally, test the perform() message (not very thoroughly)
    oSym := Symbol():name

    // now, you get 10 points for figuring out what the following line will print!
    ? oSym:perform(oSym)

    // ...it prints the word "NAME".  oSym refers to the "NAME" symbol, and
    // we use perform to send that symbol object the "NAME" message (using the
    // same symbol object) which returns the value, "NAME".

    // following line prints .f., because the two objects are not the same, even
    // though they refer to the same symbol.  This is unfortunate, but difficult
    // to correct at this stage given the architecture of Clipper.
    ? oSym == Symbol():name

    // the isEqual message allows comparison of Symbol objects
    ? oSym:isEqual( Symbol():name )         // prints .T.
    ? oSym:isEqual( Symbol():notTheSame )   // prints .F.

    // Execution of undefined names is ignored at present
    // This may change in future versions
    Symbol():notAFunction:exec()

    Symbol():__quit:exec()

    ? "Execution never reaches this point..."

RETURN


FUNCTION WhatFun(p1, p2, p3)
    LOCAL oSym := Symbol():BottomLevel

    ? oSym:name, "returned:", oSym:exec( 73 )

    // note use of Symbol:new message below
    Symbol():new("Qout"):exec("WhatFun checking in...")
RETURN p1 + p2 + p3


FUNCTION BottomLevel( p1 )
    // who needs the question mark for printing?
    Symbol():Qout:exec( "Having fun in BottomLevel..." )
RETURN p1 * 2


// eof symtest.prg
